/*
 * $Id: fpgaedit.l 18 2004-10-04 18:31:33Z bsp $
 */

%option nostack
%option nounput
%option noyywrap
%option pointer
%option nodefault
%option case-insensitive
%option prefix="zz"

%{
/* need this for the call to atol() below */
#include <math.h>
#include "y.tab.h"

extern int debug;
extern int in_line;
%}

DIGIT	[0-9]
ID	[a-z][a-z0-9]*
HEX	[0-9a-z]
OCT	[0-7]
BIN	[01]
%%

q |
x |
exit |
quit {
	if (debug > 4) printf ("EXIT\n"); 
	return EXIT;
}

h |
help {
	if (debug > 4) printf ("HELP\n"); 
	return HELP;
}

m[dm](\.[biw])* {
	if (strlen (yytext) == 2) yylval.val = 4;
	else switch (*(yytext + 3)) {
	case 'b': yylval.val = 1; break;
	case 'i': yylval.val = 4; break;
	case 'w': yylval.val = 2; break;
	}
	
	if (*(yytext + 1) == 'd') {
		if (debug > 4) printf ("MD %ld-bit\n",yylval.val*8); 
		return MD;
	}
	else { 
		if (debug > 4) printf ("MM %ld-bit\n",yylval.val*8); 
		return MM;
	}
}

\+ |
add { 
	if (debug > 4) printf ("ADD\n"); 
	return ADD;
}

\- |
sub { 
	if (debug > 4) printf ("SUB\n"); 
	return SUB;
}

\* |
mul { 
	if (debug > 4) printf ("MUL\n"); 
	return MUL;
}

\/ |
div { 
	if (debug > 4) printf ("DIV\n"); 
	return DIV;
}

\>\> |
rshift { 
	if (debug > 4) printf ("RSHIFT\n"); 
	return RSHIFT;
}

\<\< |
shift |
lshift { 
	if (debug > 4) printf ("LSHIFT\n"); 
	return LSHIFT;
}

\( { 
	if (debug > 4) printf ("OBRA\n"); 
	return OBRA;
}

\) { 
	if (debug > 4) printf ("CBRA\n"); 
	return CBRA;
}

\% |
mod { 
	if (debug > 4) printf ("MOD\n"); 
	return MOD;
}

\= |
eq { 
	if (debug > 4) printf ("EQ\n"); 
	return EQ;
}

o |
open { 
	if (debug > 4) printf ("\n"); 
	return OPEN;
}

c |
close { 
	if (debug > 4) printf ("\n"); 
	return CLOSE;
}

{ID} {
	yylval.tptr = malloc(sizeof(yytext));
	if (yylval.tptr) strcpy(yylval.tptr, yytext);
	else printf ("no mem in %s", __FUNCTION__);
	if (debug > 4) printf ("ID: >%s<\n", yytext); 
	return NAME;
}

{DIGIT}+ {
	yylval.val = atol(yytext);
	if (debug > 4) printf ("DIGIT: %s (%ld)\n", yytext, yylval.val); 
	return NUM;
}

0x{HEX}+ {
	yylval.val = strtol(yytext + 2, (char **)NULL, 16);
	if (debug > 4) printf ("HEX: %s (%ld)\n", yytext, yylval.val);
	return NUM;
}

0o{OCT}+ {
	yylval.val = strtol(yytext + 2, (char **)NULL, 8);
	if (debug > 4) printf ("OCT: %s (%ld)\n", yytext, yylval.val); 
	return NUM;
}

0b{BIN}+ {
	yylval.val = strtol(yytext + 2, (char **)NULL, 2);
	if (debug > 4) printf ("BIN: %s (%ld)\n", yytext, yylval.val);
	return NUM;
}

\n     {
	if (debug > 4) printf ("CR\n");
	in_line = 0;
	return CR;
}

.	{
	/* eat up any unmatched character */
	if (debug > 4) printf ("unmatched: 0x%02x\n", *yytext);
}
%%
